package org.omegahat.DataStructures.Data;


/**
  Class used to represent a continuous variable, rather than a factor.
*/
public class RealVariable 
         extends  AbstractVariable
          implements VariableInt, 
                     java.io.Serializable
{
  protected double[] vals;

  public RealVariable(int len) {
    vals = new double[len];
  }

  public RealVariable(int len, java.util.Hashtable metaData) {
   this(len);
   metaData(metaData);
  }

  public RealVariable(double[] d) {
    vals = d;
  }

  public RealVariable() {
  }


  public long numObservations() {
    return(vals == null ? 0 : (long)vals.length);
  }

  public Object value(int i) 
  { 
    return(new Double(vals[i])); 
  }



  public Object asArray() {
    return(vals);
  }

  public Object asArrayCopy() {
    double[] d = new double[vals.length];
    for(int i = 0; i < d.length; i++)
      d[i] = vals[i];
    return(d);
  }

  public VariableInt values(int[] ind) {
    RealVariable real = new RealVariable(ind.length);
    for(int i = 0;i < ind.length; i++)
      real.vals[i] = vals[ind[i]];
    return(real);
  }

  public String[] toStringArray() {
    String[] str = new String[vals.length];
    for(int i = 0; i < vals.length; i++)
      str[i] = String.valueOf(vals[i]);
    return(str);
  }


  /**
    Add a value as the <code>which</code> observation
   or if <code>which</code> is -1, append it to the end.
   @param value the value of the observation.
   @which the observation number or -1 indicating to append
   it to the existing collection of values.
  */
  public void add(double value, int which) {
    if(which < 0) {
      if(vals == null) {
        vals = new double[1];
      } else {
        double[] tmp = new double[vals.length + 1];
        System.arraycopy(vals, 0, tmp, 0, vals.length);
        vals = tmp;
      }
       vals[vals.length-1] = value;
    } else
     values()[which] = value;
  }


  public void add(Object value, int which) {
     double val = Double.NaN;
     if(value instanceof Number) {
       val = ((Number)value).doubleValue();
     } else {
      try {
       val = Double.parseDouble(value.toString().trim());
      } catch(NumberFormatException ex) {

      }
     }
     add(val, which);
  }
  public String toString() {
    String str = "RealVariable: # obs = " + numObservations();

   return(str);
  }

  public double[] values() {
    return(vals);
  }

  public double[] values(boolean copy) {
    double []tmp = new double[vals.length];
    System.arraycopy(tmp, 0, vals, 0, vals.length);
    return(tmp);
  }

  public double[] getValues() {
    return(values());
  }


}

