
package org.omegahat.DataStructures.Data.Viewers;

import org.omegahat.DataStructures.Data.VariableInt;
import org.omegahat.DataStructures.Data.FactorInt;

/**
 Cell Editor for the {@link DataFrameViewer DataFrameViewer}
 in columns representing {@link org.omegahat.DataStructures.Data.FactorInt FactorInt}
 variables, displaying the current levels in a choice menu and allowing
 new ones to be entered in a {@link javax.swing.JComboBox JComboBox}.
*/
public class FactorCellEditor  
        extends javax.swing.JComboBox
        implements javax.swing.table.TableCellEditor
                   
{
static final protected int STOP = 0;
static final protected int CANCEL = 1;

protected org.omegahat.DataStructures.Data.FactorInt activeVariable;

protected int activeObservation;

protected java.util.Vector listeners;

public FactorCellEditor() {
 setEditable(true);
// addActionListener(this);
}

public java.awt.Component
getTableCellEditorComponent(javax.swing.JTable table,
                               Object value, boolean isSelected, int row, int column) 
{
 if(table.getModel() instanceof DataFrameTableModel) {
  VariableInt var = ((DataFrameTableModel)table.getModel()).getVariable(column);
  if(var instanceof org.omegahat.DataStructures.Data.FactorInt) {
   Object[] levels = ((FactorInt) var).levels().levels();
    if(getItemCount() > 0)
      editor().removeAllItems();

    for(int i = 0 ; i < levels.length ; i++)
      editor().addItem(levels[i]);

    editor().setSelectedItem(value.toString());

    activeVariable((FactorInt)var);
    activeObservation(row);

    return(editor());
  }
 }

 return(new java.awt.TextField(value.toString()));
}


/**
  Add a listener to the list that's notified when the editor starts, stops, or
  cancels editing.
*/
public void addCellEditorListener(javax.swing.event.CellEditorListener l) 
{
 listeners().addElement(l);
}

/**
  Tell the editor to cancel editing and not accept any partially edited value.
*/
public void cancelCellEditing() {
   notifyListeners(CANCEL);
}

public boolean stopCellEditing() {
 notifyListeners(STOP);
 return(true);
}

/**
 Inform the listeners that the edit has been completed or cancelled,
 depending on the value of <code>type</code>
  @param type one of {@link #STOP STOP} or {@link #CANCEL CANCEL}
   indicating the type of event to report.
*/
public void notifyListeners(int type) {
  javax.swing.event.ChangeEvent ev = new javax.swing.event.ChangeEvent(this);
  for(java.util.Enumeration e = listeners().elements(); e.hasMoreElements() ;) {
   javax.swing.event.CellEditorListener l = (javax.swing.event.CellEditorListener)e.nextElement();
    if(type == STOP)
       l.editingStopped(ev);
    else
       l.editingCanceled(ev);
  } 
}


/**
                    Returns the value contained in the editor
*/
public Object getCellEditorValue() 
{
 return(editor().getSelectedItem());
}

public boolean isCellEditable(java.util.EventObject anEvent) {
 return(true);
}



public void removeCellEditorListener(javax.swing.event.CellEditorListener l) 
{
 if(listeners(false) != null && listeners(false).contains(l)) {
  listeners().remove(l);
 }
}


public boolean shouldSelectCell(java.util.EventObject anEvent) 
{
 return(true);
}


public void processKeyEvent(java.awt.event.KeyEvent ev) {
 if(ev.getKeyCode() == java.awt.event.KeyEvent.VK_ENTER)
   notifyListeners(STOP);
}
public javax.swing.JComboBox editor()
{
   return(this);
}

/**
   Accessor for {@link #activeVariable activeVariable} field
 */
public org.omegahat.DataStructures.Data.FactorInt activeVariable()
{
   return(activeVariable);
}

/**
   Accessor for setting {@link #activeVariable activeVariable} field
 */

public org.omegahat.DataStructures.Data.FactorInt activeVariable(org.omegahat.DataStructures.Data.FactorInt value)
{
   activeVariable = value;
  return(activeVariable);
}


/**
   Accessor for {@link #activeObservation activeObservation} field
 */
public int activeObservation()
{
   return(activeObservation);
}

/**
   Accessor for setting {@link #activeObservation activeObservation} field
 */

public int activeObservation(int value)
{
   activeObservation = value;
  return(activeObservation);
}

/**
   Accessor for {@link #listeners listeners} field
 */
public java.util.Vector listeners()
{
 return(listeners(true));
}

/**
   Accessor for {@link #listeners listeners} field
   that allows it to be initialized here if it has not been before this.
 */
public java.util.Vector listeners(boolean force)
{
 if(listeners == null && force) { 
  listeners(new java.util.Vector(1));
 }

  return(listeners);
}

/**
   Accessor for setting {@link #listeners listeners} field
 */

public java.util.Vector listeners(java.util.Vector value)
{
   listeners = value;
  return(listeners);
}



}

/*
Copyright (c) 1998 1999, The Omega Project for Statistical Computing.
     All rights reserved.

     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions are
     met:

       Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.

       Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

       Neither name of the Omega Project for Statistical Computing nor the 
       names of its contributors may be used to endorse or promote products 
       derived from this software without specific prior written permission.

     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY
     EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
     SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
     SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
     PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
     STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
     THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

*/

